/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.field.model;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryCollection;
import com.vividsolutions.jts.geom.Point;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import nl.cloudfarming.client.geoviewer.Geographical;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.model.AbstractEntity;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.openide.util.Exceptions;
import org.openide.util.NbBundle;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NbBundle.Messages({"ActivityField icon label=Activity field",
    "ActivityField tooltip unknown name=Unknown"})
public class ActivityField extends AbstractEntity implements Serializable, PropertyChangeListener, Geographical<Geometry>, Cloneable {

    public static final String EXT = "afd";
    private static final long serialVersionUID = 1L;
    public static final ItemIdType ITEM_ID_TYPE = ItemIdType.AFD;
    public static final String MIMETYPE = "application/x-agrosense-activityfield";
    private String name;
    public static final String PROP_NAME = "name";
    private List<TreatmentZone> treatmentZones = new ArrayList<>();
    public static final String PROP_TREATMENTZONES = "treatmentZones";
    private String description;
    public static final String PROP_DESCRIPTION = "description";
    private static final String ICON_PATH = "nl/cloudfarming/client/field/type/folder_treatmentzones.png";
    private Geometry parentGeometry;

    public ActivityField(EnterpriseIdProvider eip) {
        super(eip, ITEM_ID_TYPE);
    }

    public ActivityField(EnterpriseIdProvider eip, String name) {
        this(eip);
        this.name = name;
    }

    public ActivityField(EnterpriseIdProvider eip, String name, Geometry parentGeometry) {
        this(eip, name);
        this.parentGeometry = parentGeometry;
    }

    public ActivityField(ActivityField activityField) {
        super(activityField.getURI(), ITEM_ID_TYPE);

        setDescription(activityField.getDescription());
        setName(activityField.getName());
        List<TreatmentZone> copies = new ArrayList<>();
        for (TreatmentZone treatmentZone : activityField.getTreatmentZones()) {
            copies.add(new TreatmentZone(treatmentZone));
        }
        setTreatmentZones(copies);
        parentGeometry = activityField.getParentGeometry();
    }

    public Geometry getParentGeometry() {
        return parentGeometry;
    }

    public boolean geometryFits(Geometry geometry) {
        if (parentGeometry != null) {
            return parentGeometry.contains(geometry);
        }
        return false;
    }

    /**
     * Get the value of description
     *
     * @return the value of description
     */
    public String getDescription() {
        return description;
    }

    /**
     * Set the value of description
     *
     * @param description new value of description
     */
    public final void setDescription(String description) {
        String oldDescription = this.description;
        this.description = description;
        getPropertyChangeSupport().firePropertyChange(PROP_DESCRIPTION, oldDescription, description);
    }

    /**
     * Get the value of treatmentZones
     *
     * @return the value of treatmentZones
     */
    public List<TreatmentZone> getTreatmentZones() {
        if (treatmentZones == null) {
            treatmentZones = new ArrayList<>();
        }
        return treatmentZones;
    }

    public void addTreatmentZone(TreatmentZone treatmentZone) {
        if (!getTreatmentZones().contains(treatmentZone)) { // FIXME: change list to set; check all nested entities.
            List<TreatmentZone> oldTreatmentZones = new ArrayList<>(treatmentZones);
            this.treatmentZones.add(treatmentZone);
            treatmentZone.setParent(this);
            treatmentZone.addPropertyChangeListener(this);
            getPropertyChangeSupport().firePropertyChange(PROP_TREATMENTZONES, oldTreatmentZones, treatmentZones);
        }
    }

    public void removeTreatmentZone(TreatmentZone treatmentZone) {
        if (getTreatmentZones().contains(treatmentZone)) {
            List<TreatmentZone> oldTreatmentZones = new ArrayList<>(treatmentZones);
            this.treatmentZones.remove(treatmentZone);
            treatmentZone.setParent(null);
            treatmentZone.removePropertyChangeListener(this);
            getPropertyChangeSupport().firePropertyChange(PROP_TREATMENTZONES, oldTreatmentZones, treatmentZones);
        }
    }

    /**
     * Set the value of treatmentZones
     *
     * @param treatmentZones new value of treatmentZones
     */
    public final void setTreatmentZones(List<TreatmentZone> treatmentZones) {
        List<TreatmentZone> oldTreatmentZones = this.treatmentZones;
        if (oldTreatmentZones != null) {
            for (TreatmentZone tzn : oldTreatmentZones) {
                tzn.setParent(null);
                tzn.removePropertyChangeListener(this);
            }
        }
        
        this.treatmentZones = treatmentZones;
        if (treatmentZones != null) {
            for (TreatmentZone tzn : treatmentZones) {
                tzn.setParent(this);
                tzn.addPropertyChangeListener(this);
            }
        }
        
        getPropertyChangeSupport().firePropertyChange(PROP_TREATMENTZONES, oldTreatmentZones, treatmentZones);
    }

    /**
     * Get the value of name
     *
     * @return the value of name
     */
    public String getName() {
        return name;
    }

    /**
     * Set the value of name
     *
     * @param name new value of name
     */
    public final void setName(String name) {
        String oldName = this.name;
        this.name = name;
        getPropertyChangeSupport().firePropertyChange(PROP_NAME, oldName, name);
    }

    /**
     * Use readObject to attach listeners
     *
     * @param in
     */
    private void readObject(ObjectInputStream in) {
        try {
            in.defaultReadObject();
            for (TreatmentZone treatmentZone : treatmentZones) {
                treatmentZone.addPropertyChangeListener(this);
            }
        } catch (IOException | ClassNotFoundException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    /**
     * @param pce
     */
    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        getPropertyChangeSupport().firePropertyChange(PROP_TREATMENTZONES, null, pce.getSource());
    }

    /**
     * <p>Gets the centroid of the GeomtryCollection from the geometries in the
     * {@link TreatmentZone}'s in this ActivityField</p> <p> Constructs a
     * GeometryCollection from the {@link TreatmentZone}'s this ActivityField
     * has If there are no {@link TreatmentZone}'s or the
     * {@link TreatmentZone}'s have no geometry null is returned </p>
     *
     * @return centroid of the GeometryCollection
     */
    @Override
    public Point getCentroid() {
        GeometryCollection geometryCollection = getGeometryCollection();
        return geometryCollection != null ? geometryCollection.getCentroid() : null;
    }

    /**
     * <p>Gets the bounding box of the GeomtryCollection from the geometries in
     * the {@link TreatmentZone}'s in this ActivityField</p> <p> Constructs a
     * GeometryCollection from the {@link TreatmentZone}'s this ActivityField
     * has If there are no {@link TreatmentZone}'s or the
     * {@link TreatmentZone}'s have no geometry null is returned </p>
     */
    @Override
    public Geometry getBoundingBox() {
        GeometryCollection geometryCollection = getGeometryCollection();
        return geometryCollection != null ? geometryCollection.getEnvelope() : null;

    }

    /**
     * <p>Gets the GeomtryCollection from the geometries in the
     * {@link TreatmentZone}'s in this ActivityField</p> <p> Constructs a
     * GeometryCollection from the {@link TreatmentZone}'s this ActivityField
     * has If there are no {@link TreatmentZone}'s or the
     * {@link TreatmentZone}'s have no geometry null is returned </p>
     *
     * @return GeometryCollection
     */
    public GeometryCollection getGeometryCollection() {
        List<Geometry> geometriesList = new ArrayList<>();
        for (TreatmentZone treatmentZone : treatmentZones) {
            if (treatmentZone.getGeometry() != null) {
                geometriesList.add(treatmentZone.getGeometry());
            }
        }
        if (!geometriesList.isEmpty()) {
            Geometry[] geometries = new Geometry[geometriesList.size()];
            geometriesList.toArray(geometries);
            GeometryCollection geometryCollection = new GeometryCollection(geometries, geometries[0].getFactory());
            return geometryCollection;
        } else {
            return null;
        }
    }

    /**
     * Gets the {@link Geometrical} from {@Link ActivityField#treatmentZones}
     * Only adds {@link TreatmentZone} when it's {@link TreatmentZone#geometry}
     * is not null
     *
     * @return List of {@link Geometrical}
     */
    public List<Geometrical> getGeometricals() {
        List<Geometrical> geometriesList = new ArrayList<>();
        for (TreatmentZone treatmentZone : treatmentZones) {
            if (treatmentZone.getGeometry() != null) {
                geometriesList.add(treatmentZone);
            }
        }
        return geometriesList;
    }

    /**
     * Returns null
     *
     * @param envelope
     * @return
     */
    @Override
    public Geometry getRenderObject(Envelope envelope) {
        return null;
    }

    /**
     * Returns the tooltipText as name + description
     *
     * @return
     */
    @Override
    public String getTooltipText() {
        StringBuilder tooltipBuilder = new StringBuilder();
        if (name != null) {
            tooltipBuilder.append(name);
        } else {
            tooltipBuilder.append(Bundle.ActivityField_tooltip_unknown_name());
        }
        if (description != null) {
            tooltipBuilder.append(" ");
            tooltipBuilder.append(description);
        }
        return tooltipBuilder.toString();
    }

    @Override
    public String getIconPath() {
        return ICON_PATH;
    }

    @Override
    public String getIconLabel() {
        return Bundle.ActivityField_icon_label();
    }
}
