/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.field.model;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.DynamicGeometrical;
import nl.cloudfarming.client.lib.geotools.GeometryTools;
import nl.cloudfarming.client.model.AbstractEntity;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;
import org.openide.util.NbBundle;

/**
 * A treatment zone is a geographical area which is treated in a particular way.
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({"TreatmentZone unknown name=Unknown"})
public class TreatmentZone extends AbstractEntity implements DynamicGeometrical<Geometry>, Serializable {

    private static final long serialVersionUID = 1L;
    public static final ItemIdType ITEM_ID_TYPE = ItemIdType.TZN;
    //!! Keep the member properties always syncronised with member names. This is used in bean binding
    private Geometry geometry;
    public static final String PROP_GEOMETRY = "geometry";
    public static final String PROP_NAME = "name";
    public static final String PROP_VALUE = "value";
    public static final String PROP_AREA_IN_HECTARES = "areaInHectares";
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.task.model");
    private String name;
    private Double value;
    private double areaInHectares = 0.0d;
    public static final String EXT = "tzn";
    public static final String MIMETYPE = "application/x-agrosense-treatmentzone";
    private List<Action> mapActions;
    
    private ActivityField activityField;

    public TreatmentZone(EnterpriseIdProvider eip) {
        super(eip, ITEM_ID_TYPE);
    }

    public TreatmentZone(EnterpriseIdProvider eip, String name) {
        super(eip, ITEM_ID_TYPE);
        this.name = name;
    }

    public TreatmentZone(EnterpriseIdProvider eip, Geometry geometry) {
        super(eip, ITEM_ID_TYPE);
        setGeometry(geometry);
    }

    /**
     * Copy constructor
     *
     * @param treatmentZone The TreatmentZone to copy
     */
    public TreatmentZone(TreatmentZone treatmentZone) {
        super(treatmentZone.getURI(), ITEM_ID_TYPE);

        setName(treatmentZone.getName());
        setValue(treatmentZone.getValue());
        setGeometry(treatmentZone.getGeometry());
    }

    /**
     * Get the value of geometry
     *
     * @return the value of geometry
     */
    @Override
    public Geometry getGeometry() {
        return geometry;
    }

    /**
     * Get the bounding box of the geometry from this TreatmentZone
     *
     * @return bounding box of geometry or null if geometry is null
     */
    @Override
    public Geometry getBoundingBox() {
        return (this.geometry != null) ? geometry.getEnvelope() : null;
    }

    @Override
    public Point getCentroid() {
        return (this.geometry != null) ? geometry.getCentroid() : null;
    }

    public String getName() {
        return name;
    }

    public final void setName(String name) {
        String old = this.name;
        this.name = name;
        getPropertyChangeSupport().firePropertyChange(PROP_NAME, old, name);
    }

    public Double getValue() {
        return value;
    }

    /**
     * Set the value of this TreatmentZone Fires PropertyChangeEvent with
     * property name {@link TreatmentZone#value}
     *
     * @param value
     */
    public final void setValue(Double value) {
        Double old = this.value;
        this.value = value;
        getPropertyChangeSupport().firePropertyChange(PROP_VALUE, old, value);
    }

    /**
     * Set the area in HA of this TreatmentZone
     *
     * @param areaInHectares
     */
    public void setAreaInHectares(double areaInHectares) {
        double old = this.areaInHectares;
        this.areaInHectares = areaInHectares;
        getPropertyChangeSupport().firePropertyChange(PROP_AREA_IN_HECTARES, old, areaInHectares);
    }

    /**
     * Get the area or this TreatmentZone in HA
     *
     * @return
     */
    public double getAreaInHectares() {
        return areaInHectares;
    }

    /**
     * Set the value of geometry Also updates the {@link TreatmentZone#areaInHectares}
     *
     * @param geometry new value of geometry
     */
    @Override
    public final void setGeometry(Geometry geometry) {
        Geometry oldGeometry = this.geometry;
        this.geometry = geometry;
        if (geometry != null) {
            setAreaInHectares(GeometryTools.getAreaInHA(geometry));
        }
        getPropertyChangeSupport().firePropertyChange(PROP_GEOMETRY, oldGeometry, geometry);
    }

    @Override
    public String toString() {
        return name;
    }

    /**
     * Used to set the areaInHectares 
     * @param in
     * @throws IOException
     * @throws ClassNotFoundException 
     */
    private void readObject(java.io.ObjectInputStream in)
            throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        if (geometry != null) {
            areaInHectares = GeometryTools.getAreaInHA(geometry);
        }
    }

    @Override
    public Geometry getRenderObject(Envelope envelope) {
        return geometry;
    }

    /**
     * Get the tool tip text for this TreatmentZone
     * Will check if the name of this TreatmentZone is not null and not empty
     * if not will add the name to the tool tip
     * Will check the value for null if not the case will add the value to the tool tip
     * @return the created tool tip based on the name and value of this TreatmentZone
     */
    @Override
    public String getTooltipText() {
        String tzName = getName();
        if (getName() == null || getName().isEmpty()) {
            tzName = Bundle.TreatmentZone_unknown_name();
        }
        StringBuilder tooltipBuilder = new StringBuilder(tzName);
        if (this.getValue() != null) {
            tooltipBuilder.append(" ");
            tooltipBuilder.append(new Double(this.getValue()).toString());
        }
        return tooltipBuilder.toString();
    }

    @Override
    public String getIconPath() {
        return null;
    }

    @Override
    public String getIconLabel() {
        return null;
    }

    @Override
    public Action[] getMapActions() {
        return getActions().toArray(new Action[0]);
    }

    private List<Action> getActions() {
        if (mapActions == null) {
            mapActions = new ArrayList<>();
        }
        return mapActions;
    }

    @Override
    public void addMapAction(Action action) {
        getActions().add(action);
    }

    public ActivityField getParent() {
        return activityField;
    }

    public void setParent(ActivityField activityField) {
        this.activityField = activityField;
    }
    
    
}